#include "game.h"
#include "bitmap_manager.h"
#include "sound_manager.h"
#include "inputlib.h"
#include "stringlib.h"
#include <math.h>

list<VObject *> game_vobjects;
list<VObject *> spikes;

const int LEVEL_SIZE=10;

int level_index=0;
string level_names[LEVEL_SIZE]= {
    "data/level1.txt", "data/level2.txt", "data/level3.txt", "data/level4.txt",
    "data/level5.txt", "data/level6.txt", "data/level7.txt", "data/level8.txt",
    "data/level9.txt", "data/level10.txt"
};

VObject *player;
VObject *hat;

VObject *block;

float background_x=0;
VObject *background_bg1=NULL;
VObject *background_bg2=NULL;

const float BLOCK_W=80;
float GRAVITY=2;
float MOVE_SPEED=5;
float FALL_SPEED=0;
float MAX_FALL_SPEED=10;

float next_level_wait=120;

ALLEGRO_BITMAP *arena_border_v=NULL;
ALLEGRO_BITMAP *arena_border_h=NULL;
ALLEGRO_BITMAP *arena_bottom=NULL;

ALLEGRO_FONT *score_font;
ALLEGRO_FONT *tip_font;
ALLEGRO_FONT *warning_font;
ALLEGRO_FONT *achievement_font;

VObject *arena_blocks[12][12];
list<VObject *> falling_blocks;
list<VObject *> pushed_blocks;
list<VObject *> magic_notes;

bool To_Reset=false;


int animation_frame=0;
int animation_cooldown=0;
int score=0;

int remaining_moves=1;

void Reset_Game();

float white_flash=255;

long int Game_Time=0;

// Maximum arena width will be 12*80=960
// The arena will change from 8-10-12 blocks
// It's 12 blocks high
float arena_width=640;
float new_arena_width=640;

float arena_warning=0;
float arena_warning_dir=1;

string arena_warning_string_1="";
string arena_warning_string_2="";
string arena_warning_string_3="";

float arena_laser_x=0;
float arena_laser_y=0;
float arena_laser_charge=0;
float arena_column=0;

float bottom_start_x=0;

float player_x=300;
float player_y=0;

float MAX_JUMP_STRENGTH = 5;
float jump_strength=0;

float block_fall_speed=4;

int player_direction=0;
int jump_cooldown=10;

int hurt_cooldown=0;
int player_health=100;

float block_drop_cooldown=15;

ALLEGRO_SAMPLE_INSTANCE *beat;
int beat_change_cooldown=300;

float screen_shake=0;

int arena_change_cooldown=1000;
bool arena_spikes=0;

int total_combo_multiplier=1;

string combo_string="";
float combo_string_y;
float combo_speed = -20;
float combo_opacity = 0;
bool combo_string_active = false;

string current_level="data/level1.txt";

bool on_screen_tip=true;
int on_screen_tip_duration=0;
float on_screen_tip_opacity=0;
int on_screen_tip_cooldown=100;
string on_screen_tip_string_1="";
string on_screen_tip_string_2="";

bool first_tip=false;

list<int> used_tips;


enum {
    GAME_MODE_INFINITE,
    GAME_MODE_PUZZLE
};



// Game events happen every once in a cooldown
// Shared use with the arena cooldown timer
//
// GAME_EVENT_TOP_LASER: Laser appears on the top, moves randomly,
//        charges for 5 seconds, then erases the whole column.
// GAME_EVENT_SIDE_LASER: Same as top laser, but on the side.
//
//
//

enum {
    GAME_EVENT_NONE,
    GAME_EVENT_TOP_LASER,
    GAME_EVENT_SIDE_LASER,
    GAME_EVENT_FLOOR_SPIKES,
    GAME_EVENT_FILL_COLUMN,
    GAME_EVENT_BOMBARD,
    GAME_EVENT_RESIZE
};

int Game_Event=GAME_EVENT_NONE;
int Game_Event_Duration=0;
int Game_Mode=GAME_MODE_INFINITE;

bool Game_Created=false;



int Difficulty=100;

enum {
    ANIMATION_STAND,
    ANIMATION_WALK,
    ANIMATION_JUMP,
    ANIMATION_PUNCH
};

int player_animation=0;
int punching = 30;

float new_beat_speed=1.0;
float new_beat_gain=1.0;
float beat_speed=1.0;
float beat_gain=1.0;



string color_list[] = {
    "blue", "green", "grey", "purple", "red", "yellow", "bomb"
};

void Create_Particle_Star(float x, float y) {
    VObject *particle = new VObject(Get_Allegro_Bitmap("data/star.png"), x, y);
    particle->Set_Particle_Parameters(150+rand()%100, 200+rand()%55, (rand()%20)/10.0+1, ((rand()%360)/360.0)*ALLEGRO_PI*2, (rand()%20)/1000.0);
    particle->Set_Z(10000000);
    game_vobjects.push_back(particle);
}

void Create_Particle_Smoke(float x, float y) {
    VObject *particle = new VObject(Get_Allegro_Bitmap("data/smoke.png"), x, y);
    particle->Set_Particle_Parameters(150+rand()%100, 200+rand()%55, 0.2, ALLEGRO_PI/2, (rand()%20)/1000.0);
    particle->Set_Angle(((rand()%360)/360.0)*ALLEGRO_PI*2);
    particle->Set_Z(10000000);
    game_vobjects.push_back(particle);
}

void Create_Particle_Fire(float x, float y) {
    VObject *particle = new VObject(Get_Allegro_Bitmap("data/fire.png"), x, y);
    particle->Set_Particle_Parameters(150+rand()%100, 200+rand()%55, 0.2, ALLEGRO_PI/2, (rand()%20)/1000.0);
    particle->Set_Angle(((rand()%360)/360.0)*ALLEGRO_PI*2);
    particle->Set_Z(10000000);
    game_vobjects.push_back(particle);
}


void Create_Particle_Blade(float x, float y) {
    VObject *particle = new VObject(Get_Allegro_Bitmap("data/blade.png"), x, y);
    particle->Set_Particle_Parameters(200, 255, 20, -ALLEGRO_PI/2, 0);
    particle->Set_Z(10000000);
    game_vobjects.push_back(particle);
}

void Create_Particle_Coin(float x, float y) {
    for (int i=0; i<8; i++) {
        VObject *particle = new VObject(Get_Allegro_Bitmap("data/coin.png"), x, y);
        particle->Set_Particle_Parameters(200, 255, 10, ((float)i/8.0)*(ALLEGRO_PI*2), 0);
        particle->Set_Z(10000000);
        game_vobjects.push_back(particle);
    }
}

void Create_Particle_Heart(float x, float y) {
    VObject *particle = new VObject(Get_Allegro_Bitmap("data/heart.png"), x, y);
    particle->Set_Particle_Parameters(200, 255, 2, -ALLEGRO_PI/2, 0);
    particle->Set_Z(10000000);
    game_vobjects.push_back(particle);
}

void Show_Combo_Achievement() {
    if (total_combo_multiplier==2) combo_string = "Good!";
    if (total_combo_multiplier==3) combo_string = "Great!";
    if (total_combo_multiplier==4) combo_string = "Fantastic!";
    if (total_combo_multiplier==5) combo_string = "Amazing!";
    if (total_combo_multiplier==6) combo_string = "Incredible!";
    if (total_combo_multiplier==7) combo_string = "Puzzle master!";
    if (total_combo_multiplier==8) combo_string = "Planning master!";
    if (total_combo_multiplier>8) combo_string = "GODLIKE";

    combo_string_y = 1080;
    combo_speed = -50;
    combo_opacity = 0;
    combo_string_active = true;
}


void Drop_Blocks(int index, int row) {
    for (int y=0; y<row; y++) {
       if (arena_blocks[index][y]) {
            falling_blocks.push_back(arena_blocks[index][y]);
            arena_blocks[index][y]->Set_Trigger(true);
            arena_blocks[index][y] = NULL;
        }
    }
}



void Delete_From_List(VObject *object) {
    list<VObject *>::iterator it;
    list<VObject *> new_list;
    new_list.clear();
    for (it = game_vobjects.begin(); it!=game_vobjects.end(); it++) {
        if (object != *it) new_list.push_back(*it);
    }

    delete object;
    game_vobjects = new_list;
}

void Add_Block_Score(int type) {
    if (player_health <= 0) return;

    if (type == 0) score += 200*total_combo_multiplier;
    if (type == 1) score += 150*total_combo_multiplier;
    if (type == 2) score += 300*total_combo_multiplier;
    if (type == 3) score += 100*total_combo_multiplier;
    if (type == 4) score += 150*total_combo_multiplier;
    if (type == 5) score += 500*total_combo_multiplier;
    if (type == 6) score += 200*total_combo_multiplier;
}

list<VObject *> hurt_list;


void Trigger_Block_Destruction(int type, int index, int row, bool simulation=false) {


    float rx, ry;


    rx=arena_blocks[index][row]->Get_X();
    ry=arena_blocks[index][row]->Get_Y();


    if (!simulation) {
        screen_shake += 8+rand()%5;

        Delete_From_List(arena_blocks[index][row]);
        arena_blocks[index][row] = NULL;

        Get_Sample("data/destroy_block.ogg")->Play(0.55);

        Add_Block_Score(type);
        Create_Particle_Smoke(rx, ry);
    }
    else {
        arena_blocks[index][row]->Hurt();

        list<VObject *>::iterator it;
        for (it=hurt_list.begin(); it!=hurt_list.end(); it++) {
            if ((*it)==arena_blocks[index][row]) return;
        }
        hurt_list.push_back(arena_blocks[index][row]);
    }

    // FIRE
    if (type == 4) {
        // Search 8 corners, and trigger the event

        if (!simulation) {
            Get_Sample("data/fire.ogg")->Play(0.7);
            Create_Particle_Fire(rx, ry);
        }

        // TOP LEFT
        if (index > 0 && row > 0) {
            if (arena_blocks[index-1][row-1]) {
                if (arena_blocks[index-1][row-1]->Get_Type()==4)
                    Trigger_Block_Destruction(4, index-1, row-1, simulation);
            }
        }

        // TOP
        if (row > 0) {
            if (arena_blocks[index][row-1]) {
                if (arena_blocks[index][row-1]->Get_Type()==4)
                    Trigger_Block_Destruction(4, index, row-1, simulation);
            }
        }

        // TOP RIGHT
        if (index < 11 && row > 0) {
            if (arena_blocks[index+1][row-1]) {
                if (arena_blocks[index+1][row-1]->Get_Type()==4)
                    Trigger_Block_Destruction(4, index+1, row-1, simulation);
            }
        }

        // RIGHT
        if (index < 11) {
            if (arena_blocks[index+1][row]) {
                if (arena_blocks[index+1][row]->Get_Type()==4)
                    Trigger_Block_Destruction(4, index+1, row, simulation);
            }
        }

        // BOTTOM RIGHT
        if (index < 11 && row < 11) {
            if (arena_blocks[index+1][row+1]) {
                if (arena_blocks[index+1][row+1]->Get_Type()==4)
                    Trigger_Block_Destruction(4, index+1, row+1, simulation);
            }
        }

        // BOTTOM
        if (row < 11) {
            if (arena_blocks[index][row+1]) {
                if (arena_blocks[index][row+1]->Get_Type()==4)
                    Trigger_Block_Destruction(4, index, row+1, simulation);
            }
        }

        // BOTTOM LEFT
        if (index > 0 && row < 11) {
            if (arena_blocks[index-1][row+1]) {
                if (arena_blocks[index-1][row+1]->Get_Type()==4)
                    Trigger_Block_Destruction(4, index-1, row+1, simulation);
            }
        }

        // LEFT
        if (index > 0) {
            if (arena_blocks[index-1][row]) {
                if (arena_blocks[index-1][row]->Get_Type()==4)
                    Trigger_Block_Destruction(4, index-1, row, simulation);
            }
        }
    }

    // TREBOL
    if (type == 1) {

        if (!simulation) {
            Get_Sample("data/blade.ogg")->Play(0.7);
            Create_Particle_Blade(rx, ry);
        }

        for (int y=0; y<row; y++) {
            if (arena_blocks[index][y]) {
                if (!simulation) {
                    Create_Particle_Smoke(arena_blocks[index][y]->Get_X(), arena_blocks[index][y]->Get_Y());
                    Delete_From_List(arena_blocks[index][y]);
                    arena_blocks[index][y] = NULL;
                }
                else {
                    arena_blocks[index][y]->Hurt();
                }
            }
        }

        if (index > 0) {
            if (arena_blocks[index-1][row]) {
                if (arena_blocks[index-1][row]->Get_Type()==1)
                    Trigger_Block_Destruction(1, index-1, row, simulation);
            }
        }

        if (index < 11) {
            if (arena_blocks[index+1][row]) {
                if (arena_blocks[index+1][row]->Get_Type()==1)
                    Trigger_Block_Destruction(1, index+1, row, simulation);
            }
        }

        if (row < 11) {
            if (arena_blocks[index][row+1]) {
                if (arena_blocks[index][row+1]->Get_Type()==1)
                    Trigger_Block_Destruction(1, index, row+1, simulation);
            }
        }
    }

    // YELLOW COINS
    if (type == 5) {

        if (!simulation) {
            Create_Particle_Coin(rx, ry);
            Get_Sample("data/coin.ogg")->Play(0.7);
        }
        // Search 4 faces, and trigger the event

        // TOP
        if (row > 0) {
            if (arena_blocks[index][row-1]) {
                if (arena_blocks[index][row-1]->Get_Type()==5) {
                    Trigger_Block_Destruction(5, index, row-1, simulation);
                }
            }
        }


        // RIGHT
        if (index < 11) {
            if (arena_blocks[index+1][row]) {
                if (arena_blocks[index+1][row]->Get_Type()==5) {
                    Trigger_Block_Destruction(5, index+1, row, simulation);
                }
            }
        }


        // BOTTOM
        if (row < 11) {
            if (arena_blocks[index][row+1]) {
                if (arena_blocks[index][row+1]->Get_Type()==5) {
                    Trigger_Block_Destruction(5, index, row+1, simulation);
                }
            }
        }


        // LEFT
        if (index > 0) {
            if (arena_blocks[index-1][row]) {
                if (arena_blocks[index-1][row]->Get_Type()==5) {
                    Trigger_Block_Destruction(5, index-1, row, simulation);
                }
            }
        }
    }


    //  HEART

    if (type == 3) {
        if (!simulation) {
            if (player_health > 0) player_health+=15;
            Create_Particle_Heart(player->Get_X(), player->Get_Y());
            Get_Sample("data/heart.ogg")->Play(.7);
        }

        if (index > 0) {
            if (arena_blocks[index-1][row]) {
                if (arena_blocks[index-1][row]->Get_Type()==3)
                    Trigger_Block_Destruction(3, index-1, row, simulation);
            }
        }

        if (index < 11) {
            if (arena_blocks[index+1][row]) {
                if (arena_blocks[index+1][row]->Get_Type()==3)
                    Trigger_Block_Destruction(3, index+1, row, simulation);
            }
        }

        if (row < 11) {
            if (arena_blocks[index][row+1]) {
                if (arena_blocks[index][row+1]->Get_Type()==3)
                    Trigger_Block_Destruction(3, index, row+1, simulation);
            }
        }

        if (row > 0) {
            if (arena_blocks[index][row-1]) {
                if (arena_blocks[index][row-1]->Get_Type()==3)
                    Trigger_Block_Destruction(3, index, row-1, simulation);
            }
        }
    }



   // BLUE
    if (type == 0 && !simulation) {
        VObject *note;
        magic_notes.push_back(note=new VObject(Get_Allegro_Bitmap("data/magic_note.png"), rx, ry));
        note->Set_Z(100000);
        game_vobjects.push_back(note);
    }





    // BOMB
    if (type == 6) {
        // Search 8 corners, and trigger the event

        if (!simulation) {
            Get_Sample("data/bomb_explode.ogg")->Play(0.5);
        }

        // TOP LEFT
        if (index > 0 && row > 0) {
            if (arena_blocks[index-1][row-1]) {
                Trigger_Block_Destruction(arena_blocks[index-1][row-1]->Get_Type(), index-1, row-1, simulation);
            }
        }

        // TOP
        if (row > 0) {
            if (arena_blocks[index][row-1]) {
                Trigger_Block_Destruction(arena_blocks[index][row-1]->Get_Type(), index, row-1, simulation);
            }
        }

        // TOP RIGHT
        if (index < 11 && row > 0) {
            if (arena_blocks[index+1][row-1]) {
                Trigger_Block_Destruction(arena_blocks[index+1][row-1]->Get_Type(), index+1, row-1, simulation);
            }
        }

        // RIGHT
        if (index < 11) {
            if (arena_blocks[index+1][row]) {
                Trigger_Block_Destruction(arena_blocks[index+1][row]->Get_Type(), index+1, row, simulation);
            }
        }

        // BOTTOM RIGHT
        if (index < 11 && row < 11) {
            if (arena_blocks[index+1][row+1]) {
                Trigger_Block_Destruction(arena_blocks[index+1][row+1]->Get_Type(), index+1, row+1, simulation);
            }
        }

        // BOTTOM
        if (row < 11) {
            if (arena_blocks[index][row+1]) {
                Trigger_Block_Destruction(arena_blocks[index][row+1]->Get_Type(), index, row+1, simulation);
            }
        }

        // LEFT
        if (index > 0) {
            if (arena_blocks[index-1][row]) {
                Trigger_Block_Destruction(arena_blocks[index-1][row]->Get_Type(), index-1, row, simulation);
            }
        }

        // BOTTOM LEFT
        if (index > 0 && row < 11) {
            if (arena_blocks[index-1][row+1]) {
                Trigger_Block_Destruction(arena_blocks[index-1][row+1]->Get_Type(), index-1, row+1, simulation);
            }
        }
    }

    if (!simulation) Drop_Blocks(index, row);

    // DESTROY
}


// ALL BLOCKS EXCEPT MUSIC BLOCKS TRIGGER THE COMBO IF THERE'S 2 blocks in any of the 4 FACES, these are
// the falling blocks triggered by the destruction of one. Hence, old retro combos.

void Trigger_Combo(int index, int row) {
    bool b=false;
    int type;


    if (arena_blocks[index][row]) {

        type =arena_blocks[index][row]->Get_Type();

        if (type == 2) return;

        if (index > 0) {
            if (arena_blocks[index-1][row]) {
                if (arena_blocks[index-1][row]->Get_Type()==type) {
                    total_combo_multiplier += 1;

                    Trigger_Block_Destruction(type, index-1, row);
                    b = true;
                }
            }
        }

        if (index < 11) {
            if (arena_blocks[index+1][row]) {
                if (arena_blocks[index+1][row]->Get_Type()==type) {
                    total_combo_multiplier += 1;

                    Trigger_Block_Destruction(type, index+1, row);
                    b = true;
                }
            }
        }

        if (row < 11) {
            if (arena_blocks[index][row+1]) {
                if (arena_blocks[index][row+1]->Get_Type()==type) {
                    total_combo_multiplier += 1;

                    Trigger_Block_Destruction(type, index, row+1);
                    b = true;
                }
            }
        }

        if (row > 0) {
            if (arena_blocks[index][row-1]) {
                if (arena_blocks[index][row-1]->Get_Type()==type) {
                    total_combo_multiplier += 1;

                    Trigger_Block_Destruction(type, index, row-1);
                    b = true;
                }
            }
        }
    }

    if (arena_blocks[index][row] && b) {
        Trigger_Block_Destruction(arena_blocks[index][row]->Get_Type(), index, row);
    }
}



// Object-to-object bounding-box collision detector:
bool Box_Collision(VObject *object1, VObject *object2) {

	int left1, left2;
	int right1, right2;
	int top1, top2;
	int bottom1, bottom2;

	left1 = object1->Get_X()  - object1->Get_W()/2;
	left2 = object2->Get_X()  - object2->Get_W()/2;
	right1 = object1->Get_X() + object1->Get_W()/2;
	right2 = object2->Get_X() + object2->Get_W()/2 - 17;

	top1 = object1->Get_Y()    - object1->Get_H()/2;
	top2 = object2->Get_Y()    - object2->Get_H()/2 + 17;
	bottom1 = object1->Get_Y() + object1->Get_H()/2;
	bottom2 = object2->Get_Y() + object2->Get_H()/2;

	if (bottom1 < top2) return false;
	if (top1 > bottom2) return false;

	if (right1 < left2) return false;
	if (left1 > right2) return false;

	return true;

};



VObject *Create_Block(float x, float y, int index, int color) {
    VObject *block;
    ALLEGRO_BITMAP *bit;
    bit = Get_Allegro_Bitmap("data/block_" + color_list[color]+".png");
    block = new VObject(bit, x, y);
    block->Set_Z(x);
    block->Set_Index(index);
    block->Set_Type(color);
    if (color == 0) block->Set_Health(3);
    if (color == 1) block->Set_Health(3);
    if (color == 2) block->Set_Health(16);
    if (color == 3) block->Set_Health(3);
    if (color == 4) block->Set_Health(3);
    if (color == 5) block->Set_Health(3);
    if (color == 6) block->Set_Health(3);

    return block;
}



bool Player_Check_Collision(float new_x, float new_y) {
    float old_x, old_y;
    list<VObject *>::iterator it;
    old_x = player->Get_X();
    old_y = player->Get_Y();

    if (new_x <= (960-arena_width)/2 + BLOCK_W/2-20) {
        return true;
    }

    if (new_x >= (960-arena_width)/2+arena_width - BLOCK_W/2+20) {
        return true;
    }


    if (new_y > 960) return true;
    if (new_y < 40) return true;

    player->Set_X(renderer->RXtoX(renderer->Get_Width()/2)-BLOCK_W*6+new_x);
    player->Set_Y(new_y);

    hat->Set_X(player->Get_X());
    hat->Set_Y(player->Get_Y());


    for (it = falling_blocks.begin(); it!=falling_blocks.end(); it++)
    if (Box_Collision(player, *it)) {
        player->Set_X(old_x);
        player->Set_Y(old_y);
        return true;
    }


    for (int x=0; x<12; x++) {
        for (int y=0; y<12; y++) {
            if (arena_blocks[x][y]) if (Box_Collision(player, arena_blocks[x][y])) {
                player->Set_X(old_x);
                player->Set_Y(old_y);
                return true;
            }
        }
    }

    player->Set_X(old_x);
    player->Set_Y(old_y);

    return false;
}


void Hurt_Player() {
    if (player_health <= 0) return;

    hurt_cooldown = 40;
    player->Hurt();
    player_health -= 30;

    if (player_health <= 0) {
        player_health = 0;
        new_beat_gain=0.1;
        new_beat_speed=0;

        player->Hide();
        hat->Hide();
        for (int i=0; i<16; i++) Create_Particle_Star(player->Get_X(), player->Get_Y());
    }

    Get_Sample("data/player_hit.ogg")->Play();
}

void Update_Game() {
    int index;
    int new_row=0;
    list<VObject *>::iterator it;

    if (white_flash > 0.0) white_flash -= 4.0;
    if (white_flash < 0) white_flash = 0.0;


    // DIFFICULTY CALCULATOR


    if (Game_Mode == GAME_MODE_INFINITE) {
        Difficulty = 1000;
        for (int x=0; x<12; x++) {
            for (int y=0; y<12; y++) {
                if (arena_blocks[x][y]) Difficulty-=20;
            }
        }

        Difficulty += Game_Time/60.0;
        Game_Time += 1;

        Difficulty += player_health;
    }
    else {
        Difficulty = 500;
    }

    if (Difficulty < 1) Difficulty = 1;


    // PLAYER CHARACTER MOVEMENT

    animation_cooldown -= 1;
    if (animation_cooldown < 0) {
        animation_frame+=1;
        animation_cooldown=10;

        if (animation_frame > 3) animation_frame = 0;
    }

    player_animation = ANIMATION_STAND;

    float new_x, new_y;
    new_x = player_x;
    new_y = player_y;

    if (hurt_cooldown > 0) hurt_cooldown -= 1;

    if (Player_Check_Collision(new_x, new_y)) {
        player_y -= 15;

        if (hurt_cooldown <= 0) {
          Hurt_Player();
        }
    }

    if (arena_spikes && player_y > 940)  {
        if (hurt_cooldown <= 0) {
          Hurt_Player();
        }
    }


    if (KB_Active[ALLEGRO_KEY_LEFT]) {
        new_x -= MOVE_SPEED;
        player_direction = 0;
        if (!Player_Check_Collision(new_x, new_y)) player_x = new_x;

        player_animation = ANIMATION_WALK;
    }
    if (KB_Active[ALLEGRO_KEY_RIGHT]) {
        new_x += MOVE_SPEED;
        player_direction = ALLEGRO_FLIP_HORIZONTAL;
        if (!Player_Check_Collision(new_x, new_y)) player_x = new_x;

        player_animation = ANIMATION_WALK;
    }

    FALL_SPEED += GRAVITY;
    if (FALL_SPEED < -MAX_FALL_SPEED*3) FALL_SPEED = -MAX_FALL_SPEED*3;
    if (FALL_SPEED > MAX_FALL_SPEED) FALL_SPEED = MAX_FALL_SPEED;

    new_y += FALL_SPEED;
    if (!Player_Check_Collision(player_x, new_y)) {
        player_y = new_y;
        player_animation = ANIMATION_JUMP;
    }
    else {
        FALL_SPEED = 0;

        if (KB_Active[ALLEGRO_KEY_UP] && Player_Check_Collision(player_x, player_y+16) && jump_cooldown<=0 && player_health > 0) {
            FALL_SPEED = -MAX_FALL_SPEED*3;
            Get_Sample("data/jump.ogg")->Play(0.7);
            jump_cooldown = 10;
        }
    }

    if (jump_cooldown > 0) jump_cooldown -= 1;

    // BLOCK PUNCHING

    punching -= 1;
    if (punching <= 0 && (player_health > 0)) {
        punching = 0;
        if (KB_Input[ALLEGRO_KEY_SPACE]) {
            punching = 20;
            animation_frame = 0;
            animation_cooldown = 5;
            Get_Sample("data/punch.ogg")->Play(0.8);

            int index, row;
            index = player_x / BLOCK_W + (player_direction ? 1 : -1);
            row = (player_y-BLOCK_W/2)/BLOCK_W;

            if (index < 0) index = 0;
            if (index > 11) index = 11;

            if (row < 0) row = 0;
            if (row > 11) row = 11;

            if (arena_blocks[index][row]) {
                // is it pushed or damaged?
                int pindex=index+(player_direction ? 1 : -1);
                bool push = true;
                if (pindex < 6-(arena_width/BLOCK_W/2)) push = false;
                else if (pindex >= 6+(arena_width/BLOCK_W/2)) push = false;
                else if (arena_blocks[pindex][row]) push = false;

                // Search for nearby falling blocks
                if (push) {
                    for (it = falling_blocks.begin(); it!=falling_blocks.end(); it++) {
                        if ((*it)->Get_Index() == pindex) {
                            if (abs((int)((*it)->Get_Y()-(row*BLOCK_W))) < BLOCK_W*1.5) push=false;
                        }
                    }
                }

                if (push) {
                    pushed_blocks.push_back(arena_blocks[index][row]);
                    arena_blocks[index][row]->Set_Direction(player_direction);
                    arena_blocks[index][row]=NULL;

                    for (int y=0; y<row; y++) {
                        if (arena_blocks[index][y]) {
                            falling_blocks.push_back(arena_blocks[index][y]);
                            arena_blocks[index][y] = NULL;
                        }
                    }
                }

                if (!push) {
                    total_combo_multiplier = 1;
                    arena_blocks[index][row]->Hurt();
                    arena_blocks[index][row]->Reduce_Health();

                    if (arena_blocks[index][row]->Get_Health() <= 0 && (remaining_moves > 0)) {
                        Trigger_Block_Destruction(arena_blocks[index][row]->Get_Type(), index, row);
                        if (Game_Mode == GAME_MODE_PUZZLE) remaining_moves-=1;
                    }
                    else {
                        hurt_list.clear();
                        Trigger_Block_Destruction(arena_blocks[index][row]->Get_Type(), index, row, true);
                    }
                }
            }
        }
    }
    else {
        player_animation = ANIMATION_PUNCH;
    }



    // Sprite movement
    background_x += beat_speed*3.5;
    if (background_x >= 1920) background_x-=1920;

    background_bg1->Set_X(background_x);
    if (background_x >= 960) background_bg2->Set_X(background_x-1920);
    else background_bg2->Set_X(background_x+1920);

    player->Set_X(renderer->RXtoX(renderer->Get_Width()/2)-BLOCK_W*6+player_x);
    player->Set_Y(player_y);
    player->Set_Flags(player_direction);

    hat->Set_Flags(player_direction);

    if (player_animation == ANIMATION_STAND) {
        string name;
        name = "1";
        player->Set_Actual_Bit(Get_Allegro_Bitmap("data/character/stand/"+name+".png"));
    }

    if (player_animation == ANIMATION_WALK) {
        string name;
        if (animation_frame == 3) name = "2";
        else name = ToString(animation_frame+1);

        player->Set_Actual_Bit(Get_Allegro_Bitmap("data/character/walk/"+name+".png"));
    }

    if (player_animation == ANIMATION_JUMP) {
        string name;
        name = "1";
        player->Set_Actual_Bit(Get_Allegro_Bitmap("data/character/jump/"+name+".png"));
    }

    if (player_animation == ANIMATION_PUNCH) {
        string name;
        if (animation_frame == 0) name = "1";
        else name = "2";
        player->Set_Actual_Bit(Get_Allegro_Bitmap("data/character/punch/"+name+".png"));
    }



    // Block generation
    block_drop_cooldown -= 1;
    if (block_drop_cooldown <= 0 && Game_Mode == GAME_MODE_INFINITE) {
        float x, y;
        int color;
        int index;
        if (Game_Event == GAME_EVENT_FILL_COLUMN) index = arena_column;
        else index = (12-(arena_width/BLOCK_W))/2 + rand()%(int)(arena_width/BLOCK_W);

        color = 250+rand()%4000 - Difficulty;
        if (color < 250) color = 2;
        if (color >= 250 && color < 1000) color = 0;
        if (color >= 1000 && color < 1500) color = 3;
        if (color >= 1500 && color < 2000) color = 1;
        if (color >= 2000 && color < 2500) color = 4;
        if (color >= 2500 && color < 3000) color = 6;
        if (color > 3000) color = 5;

        if (Game_Event == GAME_EVENT_BOMBARD) color = 6;

        if (!arena_blocks[index][0]) {
            x = renderer->RXtoX(renderer->Get_Width()/2)-BLOCK_W*6+(index)*BLOCK_W + BLOCK_W/2;
            y = -40;

            block = Create_Block(x, y, index, color);

            falling_blocks.push_back(block);
            game_vobjects.push_back(block);
        }

        block_drop_cooldown = 120 - beat_speed*40;
    }



    // Block Falling

    bool can_combo=true;

    list<VObject *> new_list;
    new_list.clear();

    for (it = falling_blocks.begin(); it!=falling_blocks.end(); it++) {
        float new_y;

        bool b=false;

        if ((*it)->Get_Trigger()) can_combo=false;

        index = (*it)->Get_Index();


        new_y = (*it)->Get_Y();
        new_y += block_fall_speed+beat_speed;

        new_row = (new_y) / BLOCK_W;
        if (new_row >= 11) new_row = 11;
        if (new_row <= 1) new_row = 1;

        if (arena_blocks[index][new_row]) {
            new_y = (new_row)*BLOCK_W;
            arena_blocks[index][new_row-1] = *it;
            b = true;
        }

        if (new_y >= 960) {
            new_y = 960;
            arena_blocks[index][11] = *it;
            b = true;
        }

        (*it)->Set_Y(new_y);
        (*it)->Set_Z((*it)->Get_X()+960-new_y);

        if (Box_Collision(player, *it)) {
            player_y += block_fall_speed;
            player->Set_Y(player_y);
        }

        if (!b) new_list.push_back(*it);
    }

    falling_blocks = new_list;


    // COMBO LIST UPDATE

    static int last_multiplier=0;

    if (can_combo) {
        for (int x=0; x<12; x++) {
            for (int y=0; y<12; y++) {
                if (arena_blocks[x][y]) {
                    if (arena_blocks[x][y]->Get_Trigger()) {
                        arena_blocks[x][y]->Set_Trigger(false);
                        Trigger_Combo(x, y);
                    }
                }
            }
        }

        if (total_combo_multiplier > 1 && (total_combo_multiplier != last_multiplier)) Show_Combo_Achievement();
    }
    last_multiplier = total_combo_multiplier;



    // PUSHING BLOCKS
    new_list.clear();
    for (it = pushed_blocks.begin(); it!=pushed_blocks.end(); it++) {
        float new_x, target_x;
        bool b=false;
        int row;
        index = (*it)->Get_Index();


        new_x = (*it)->Get_X();
        new_x += block_fall_speed*2*((*it)->Get_Direction() ? 1 : -1);

        target_x=renderer->RXtoX(renderer->Get_Width()/2)-BLOCK_W*6+(index+((*it)->Get_Direction() ? 1 : -1))*BLOCK_W + BLOCK_W/2;


        if (abs((int)(target_x - new_x)) < 4) {
            index += ((*it)->Get_Direction() ? 1 : -1);
            new_x = target_x;
            row = (int)((*it)->Get_Y()/BLOCK_W)-1;
            (*it)->Set_Index(index);
            b = true;

            if (row == 11) arena_blocks[index][row] = *it;
            else if (arena_blocks[index][row+1]) arena_blocks[index][row] = *it;
            else falling_blocks.push_back(*it);
        }

        (*it)->Set_X(new_x);
        (*it)->Set_Z(new_x+960-(*it)->Get_Y());
        if (!b) new_list.push_back(*it);
    }

    pushed_blocks = new_list;


    // MAGICAL NOTES UPDATING

    new_list.clear();
    for (it = magic_notes.begin(); it!=magic_notes.end(); it++) {
        bool b=false;

        VObject *target=NULL;
        float winner_distance=9999999;
        for (int x=0; x<12; x++) {
            for (int y=0; y<12; y++) {
                if (arena_blocks[x][y]) {
                    if (arena_blocks[x][y]->Get_Type() == 0) {
                        float dx, dy, dist;
                        dx = (*it)->Get_X() - arena_blocks[x][y]->Get_X();
                        dy = (*it)->Get_Y() - arena_blocks[x][y]->Get_Y();
                        dist=sqrt(dx*dx+dy*dy);
                        if (dist < BLOCK_W*8) {
                            if (dist <= winner_distance) {
                                winner_distance = dist;
                                target = arena_blocks[x][y];
                            }
                        }
                    }
                }
            }
        }

        if (!target) {
            Delete_From_List(*it);
        }
        else {
            new_list.push_back(*it);

            float dx, dy, dist;
            dx = target->Get_X()-(*it)->Get_X();
            dy = target->Get_Y()-(*it)->Get_Y();

            (*it)->Set_X((*it)->Get_X()+dx/10.0);
            (*it)->Set_Y((*it)->Get_Y()+dy/10.0);

            (*it)->Set_Angle((*it)->Get_Angle()+0.12);
            Create_Particle_Star((*it)->Get_X(), (*it)->Get_Y());

            if (dist=sqrt(dx*dx+dy*dy) < BLOCK_W/4) {
                arena_blocks[target->Get_Index()][(int)(target->Get_Y()/BLOCK_W)-1] = NULL;
                Drop_Blocks(target->Get_Index(), (int)(target->Get_Y()/BLOCK_W)-1);
                Delete_From_List(target);

                for (int i=0; i<30+rand()%10; i++) Create_Particle_Star((*it)->Get_X(), (*it)->Get_Y());
                Get_Sample("data/magic_note_bounce.ogg")->Play(0.7, 0.0, 1.0);

                Create_Particle_Smoke(target->Get_X(), target->Get_Y());
            }
        }
    }

    magic_notes = new_list;



    // MUSIC GENERATION

    beat_change_cooldown-=1;
    if (beat_change_cooldown <= 0 && (player_health > 0)) {
        beat_change_cooldown = 300 + rand()%1000;

        new_beat_gain = 0.7+(rand()%30)/100.0;
        new_beat_speed = 0.1+(rand()%200)/100.0 + Difficulty/2000.0;
        if (new_beat_speed > 1.8) new_beat_speed = 1.8;


    }

    beat_gain += (new_beat_gain-beat_gain)/60;
    beat_speed += (new_beat_speed-beat_speed)/60;

    al_set_sample_instance_gain(beat, beat_gain);
    al_set_sample_instance_speed(beat, beat_speed);


    // Updating VObjects and deleting particles

    new_list.clear();
    for (it = game_vobjects.begin(); it!=game_vobjects.end(); it++) {
        (*it)->Update();
        if ((*it)->To_Delete()) delete (*it);
        else new_list.push_back(*it);
    }
    game_vobjects = new_list;

    screen_shake = screen_shake*0.91;

    // COMBO STRING UPDATE

    if (combo_string_active) {
        if (combo_speed < 0) {
            combo_string_y += (400-combo_string_y)/10;
            combo_opacity += (255-combo_opacity)/10;

            if (abs((int)(400-combo_string_y)) < 4) {
                combo_speed = 1;
            }
        }
        else {
            combo_string_y += (1080-combo_string_y)/10;
            combo_opacity += (-combo_opacity)/20;

            if (combo_opacity < 10) combo_string_active = false;
        }
    }


    // ARENA CHANGE

    if (Game_Mode == GAME_MODE_INFINITE) {
        arena_change_cooldown -= 1;

        if (Game_Event_Duration > 0) Game_Event_Duration -= 1;
        if (Game_Event_Duration <= 0) {
            Game_Event = GAME_EVENT_NONE;


            for (it=spikes.begin(); it!=spikes.end(); it++) {
                (*it)->Hide();
            }

            if (arena_spikes) {
                screen_shake = 15;
                arena_spikes = false;
            }
        }
    }

    if (arena_change_cooldown < 300 && Game_Mode == GAME_MODE_INFINITE) {
        if (Game_Event == GAME_EVENT_NONE) {
            Game_Event = rand()%6000;


            if (Game_Event >= 0 && Game_Event < 1000) {
                Game_Event = GAME_EVENT_BOMBARD;
                arena_warning_string_1 = "BOMBS INCOMING";
                arena_warning_string_2 = "USE WISELY";
                arena_warning_string_3 = "";
                Game_Event_Duration = 600;
            }
            if (Game_Event >= 1000 && Game_Event < 2500) {
                Game_Event = GAME_EVENT_RESIZE;

                arena_warning_string_1 = "STAGE MODIFICATION";
                arena_warning_string_2 = "IN PROGRESS";
                arena_warning_string_3 = "GET OFF THE SIDES";

                Game_Event_Duration = 600;
            }

            if (Game_Event >= 2500 && Game_Event < 3500) {
                Game_Event = GAME_EVENT_SIDE_LASER;

                arena_warning_string_1 = "WALL LASER";
                arena_warning_string_2 = "CHARGING";
                arena_warning_string_3 = "AVOID";

                arena_laser_y = 2+rand()%8;
                arena_laser_charge=1;

                Game_Event_Duration = 600;
            }

            if (Game_Event >= 3500 && Game_Event < 4500) {
                Game_Event = GAME_EVENT_TOP_LASER;

                arena_warning_string_1 = "ROOF LASER";
                arena_warning_string_2 = "CHARGING";
                arena_warning_string_3 = "AVOID";

                arena_laser_x = 2+rand()%8;
                arena_laser_charge=1;

                Game_Event_Duration = 600;
            }

            if (Game_Event >= 4500 && Game_Event < 5500)  {
                Game_Event = GAME_EVENT_FLOOR_SPIKES;

                arena_warning_string_1 = "SPIKES PIT";
                arena_warning_string_2 = "STAY AWAY";
                arena_warning_string_3 = "";

                Game_Event_Duration = 1000;
            }

            if (Game_Event >= 5500) {
                Game_Event = GAME_EVENT_FILL_COLUMN;

                arena_warning_string_1 = "PILLAR INCOMING";
                arena_warning_string_2 = "MOVE OUT";
                arena_warning_string_3 = "";

                arena_column=2+rand()%8;

                Game_Event_Duration = 600;
            }



        }

        arena_warning += 5*arena_warning_dir;
        if (arena_warning_dir < 0 && arena_warning < 0) {
            arena_warning = 0;
            arena_warning_dir *= -1;
        }
        if (arena_warning_dir > 0 && arena_warning > 200) {
            arena_warning = 200;
            arena_warning_dir *= -1;
        }
    }

    if ((Game_Event == GAME_EVENT_SIDE_LASER) || (Game_Event == GAME_EVENT_TOP_LASER)) {
        arena_laser_charge += 0.2;
    }
    else if (arena_laser_charge>0) arena_laser_charge=arena_laser_charge*0.95;

    if (arena_laser_charge<0.1) {
        arena_laser_charge=0;
        arena_laser_x=0;
        arena_laser_y=0;
    }

    if (arena_change_cooldown <= 0 && Game_Mode == GAME_MODE_INFINITE) {
        arena_change_cooldown = 800+rand()%400;
        arena_warning = 0;

        if (Game_Event == GAME_EVENT_RESIZE) {
            int ind;
            ind = 8+(rand()%3)*2;
            new_arena_width = BLOCK_W*ind;
        }

        if (Game_Event == GAME_EVENT_TOP_LASER) {
            Get_Sample("data/laser.ogg")->Play();
            total_combo_multiplier=1;

            for (int y=0; y<12; y++) {
                if (arena_blocks[(int)arena_laser_x][y]) Trigger_Block_Destruction(arena_blocks[(int)arena_laser_x][y]->Get_Type(), arena_laser_x, y);
            }

            if (abs((int)(player_x - (arena_laser_x*BLOCK_W + BLOCK_W/2))) < BLOCK_W) {
                Hurt_Player(); Hurt_Player(); Hurt_Player();
            }

            Game_Event = GAME_EVENT_NONE;
        }

        if (Game_Event == GAME_EVENT_SIDE_LASER) {
            Get_Sample("data/laser.ogg")->Play();
            total_combo_multiplier=1;

            for (int x=0; x<12; x++) {
                if (arena_blocks[x][(int)arena_laser_y]) Trigger_Block_Destruction(arena_blocks[x][(int)arena_laser_y]->Get_Type(), x, (int)arena_laser_y);
            }

            if (abs((int)(player_y - (arena_laser_y*BLOCK_W + BLOCK_W))) < BLOCK_W) {
                Hurt_Player(); Hurt_Player(); Hurt_Player();
            }

            Game_Event = GAME_EVENT_NONE;
        }

        if (Game_Event == GAME_EVENT_BOMBARD) {
            Game_Event = GAME_EVENT_NONE;
        }

        if (Game_Event == GAME_EVENT_FILL_COLUMN) {
            Game_Event = GAME_EVENT_NONE;
        }

        if (Game_Event == GAME_EVENT_FLOOR_SPIKES) {
            total_combo_multiplier=1;
            Get_Sample("data/spike.ogg")->Play();

            for (it=spikes.begin(); it!=spikes.end(); it++) {
                (*it)->Show();
            }
            screen_shake = 30;

            for (int x=0; x<12; x++) {
                if (arena_blocks[x][11]) Trigger_Block_Destruction(arena_blocks[x][11]->Get_Type(), x, 11);
            }

            arena_spikes=true;
        }
    }

    arena_width += (new_arena_width-arena_width)*0.05;
    if (abs((int)(new_arena_width-arena_width)) < 5) arena_width = new_arena_width;

    if (player_health > 400) player_health = 400;

    if ((player_health <= 0 || Game_Mode == GAME_MODE_PUZZLE) && KB_Input[ALLEGRO_KEY_R]) {
        Reset_Game();
    }

    if (KB_Input[ALLEGRO_KEY_ESCAPE]) {
        Scene = SCENE_MENU;
        To_Reset=true;
        return;
    }

    // GAME MODE PUZZLE UPDATE
    if (Game_Mode == GAME_MODE_PUZZLE) {
        bool clean=true;

        for (int x=0; x<12; x++) {
            for (int y=0; y<12; y++) {
                if (arena_blocks[x][y]) {
                    clean=false;
                    break;
                }
            }
        }

        if (clean & (falling_blocks.size() == 0)) {
            next_level_wait -= 1;
        }
    }

    if (next_level_wait <= 0) {
        level_index += 1;

        if (level_index >= LEVEL_SIZE) {
            Scene = SCENE_MENU;
            level_index = 0;
        }
        else {
            current_level = level_names[level_index];
            Reset_Game();
        }

    }

    // UPDATE ONSCREEN TIP

    on_screen_tip_cooldown -= 1;

    if (on_screen_tip_cooldown <= 0) {
        on_screen_tip_cooldown = 1000+rand()%1000;
        on_screen_tip_opacity = 0;
        on_screen_tip = true;
        on_screen_tip_duration = 300;


        int tip=0;

        bool b=true;
        if (used_tips.size() >= 21) used_tips.clear();

        while (1) {
            b = true;
            tip = 1+rand()%21;
            list<int>::iterator it;
            for (it=used_tips.begin(); it!=used_tips.end(); it++) {
                if (tip == *it) {
                    b = false;
                }
            }

            if (b) break;
        }
        if (first_tip) {
            tip = 0;
            first_tip = false;
        }

        if (tip == 0) {
            on_screen_tip_string_1="Use the arrow keys to move";
            on_screen_tip_string_2="Press SPACE to push and destroy blocks";
        }

        if (tip == 1) {
            on_screen_tip_string_1="Fire blocks do chain reactions";
            on_screen_tip_string_2="in all 8 directions.";
        }

        if (tip == 2) {
            on_screen_tip_string_1="Break heart blocks to recover";
            on_screen_tip_string_2="part of your health.";
        }


        if (tip == 3) {
            on_screen_tip_string_1="Spikes and lasers will trigger";
            on_screen_tip_string_2="the blocks as if you had hit them.";
        }


        if (tip == 4) {
            on_screen_tip_string_1="Try to start combo chains";
            on_screen_tip_string_2="to increase your score multiplier.";
        }


        if (tip == 5) {
            on_screen_tip_string_1="Good gamers deserve Allegro 5.";
            on_screen_tip_string_2="You should switch to Allegro 5.";
        }


        if (tip == 6) {
            on_screen_tip_string_1="Allegro 5 can do this alpha blending";
            on_screen_tip_string_2="with 3D acceleration by default!";
        }


        if (tip == 7) {
            on_screen_tip_string_1="Spade(green) blocks will erase all";
            on_screen_tip_string_2="blocks that are above it.";
        }


        if (tip == 8) {
            on_screen_tip_string_1="This is powered by the magic of";
            on_screen_tip_string_2="ALLEGRO 5";
        }


        if (tip == 9) {
            on_screen_tip_string_1="Allegro 5 makes me flirty.";
            on_screen_tip_string_2="Allegro 5 makes me happy.";
        }


        if (tip == 10) {
            on_screen_tip_string_1="Allegro 5 beats SDL";
            on_screen_tip_string_2="and XNA is a rip-off.";
        }


        if (tip == 11) {
            on_screen_tip_string_1="Bombs not only erase the 8 blocks";
            on_screen_tip_string_2="around it, it also triggers them!";
        }


        if (tip == 12) {
            on_screen_tip_string_1="Blocks can only be damaged if";
            on_screen_tip_string_2="there is no more room to push them.";
        }


        if (tip == 13) {
            on_screen_tip_string_1="Anything that is marked red on the screen";
            on_screen_tip_string_2="is likely to be dangerous.";
        }

        if (tip == 14) {
            on_screen_tip_string_1="When floor spikes are about to appear";
            on_screen_tip_string_2="make sure there are no Spade blocks below.";
        }


        if (tip == 15) {
            on_screen_tip_string_1="Allegro 5 is more compatible than before!";
            on_screen_tip_string_2="You can even use a 3D engine with it.";
        }

        if (tip == 16) {
            on_screen_tip_string_1="If Allegro 5 is here, why would you";
            on_screen_tip_string_2="ever use Allegro 4.4...";
        }


        if (tip == 17) {
            on_screen_tip_string_1="Allegro 5 even supports pixel shaders,";
            on_screen_tip_string_2="how can you not use it?";
        }


        if (tip == 18) {
            on_screen_tip_string_1="Keep a distance of at least 2 blocks";
            on_screen_tip_string_2="from an incoming laser.";
        }


        if (tip == 19) {
            on_screen_tip_string_1="TIP #19: ALLEGRO 5";
            on_screen_tip_string_2="is filled with AWESOMENESS";
        }

        if (tip == 20) {
            on_screen_tip_string_1="Whenever you're in doubt";
            on_screen_tip_string_2="hit SPACE and JUMP";
        }

        if (tip == 21) {
            on_screen_tip_string_1="Coin blocks yield more points";
            on_screen_tip_string_2="than the rest.";
        }


        used_tips.push_back(tip);
    }

    if (on_screen_tip_duration > 0) on_screen_tip_duration -= 1;
    if (on_screen_tip_duration <= 0) on_screen_tip = false;

    if (on_screen_tip) on_screen_tip_opacity += (255.0-on_screen_tip_opacity)/10.0;
    else on_screen_tip_opacity += (-on_screen_tip_opacity)/10.0;


}



void Draw_Game() {
    float cx, cy, angle;
    angle = rand()%360;
    angle = angle/360.0 * ALLEGRO_PI*2;
    cx = cos(angle)*screen_shake;
    cy = sin(angle)*screen_shake;

    renderer->Render(&game_vobjects, cx, cy);

    al_draw_filled_rectangle(0, 0, renderer->Get_Width()/2 - renderer->XtoRX(arena_width/2), renderer->YtoRY(1080), al_map_rgba(0, 0, 0, 175));
    al_draw_filled_rectangle(renderer->Get_Width()/2 + renderer->XtoRX(arena_width/2), 0, renderer->XtoRX(1920), renderer->YtoRY(1080), al_map_rgba(0, 0, 0, 175));

    al_draw_filled_rectangle(renderer->Get_Width()/2 - renderer->XtoRX(arena_width/2), 0, renderer->Get_Width()/2 + renderer->XtoRX(arena_width/2), renderer->YtoRY(40), al_map_rgba(0, 0, 0, 175));
    al_draw_filled_rectangle(renderer->Get_Width()/2 - renderer->XtoRX(arena_width/2), renderer->YtoRY(1040), renderer->Get_Width()/2 + renderer->XtoRX(arena_width/2), renderer->YtoRY(1080), al_map_rgba(0, 0, 0, 175));

    if (arena_warning >= 0 && arena_change_cooldown < 300) {

        if (Game_Event == GAME_EVENT_RESIZE) {
            al_draw_filled_rectangle(renderer->Get_Width()/2 - renderer->XtoRX(arena_width/2), renderer->YtoRY(40), renderer->Get_Width()/2 - renderer->XtoRX(320), renderer->YtoRY(1040), al_map_rgba(255, 50, 50, arena_warning));
            al_draw_filled_rectangle(renderer->Get_Width()/2 + renderer->XtoRX(320), renderer->YtoRY(40), renderer->Get_Width()/2 + renderer->XtoRX(arena_width/2), renderer->YtoRY(1040), al_map_rgba(255, 50, 50, arena_warning));
        }

        if (Game_Event == GAME_EVENT_TOP_LASER) {
            cx = renderer->Get_Width()/2 - renderer->XtoRX(480) + renderer->XtoRX(arena_laser_x*BLOCK_W);
            al_draw_filled_rectangle(cx, renderer->YtoRY(40), cx+renderer->XtoRX(BLOCK_W), renderer->YtoRY(1040), al_map_rgba(255, 50, 50, arena_warning/2));

            al_draw_filled_circle(cx+renderer->XtoRX(BLOCK_W/2), renderer->YtoRY(40), renderer->YtoRY(arena_laser_charge), al_map_rgba(200+arena_warning/4, 250, 100, 175));
        }

        if (Game_Event == GAME_EVENT_FILL_COLUMN) {
            cx = renderer->Get_Width()/2 - renderer->XtoRX(480) + renderer->XtoRX(arena_column*BLOCK_W);
            al_draw_rectangle(cx, renderer->YtoRY(40), cx+renderer->XtoRX(BLOCK_W), renderer->YtoRY(1040), al_map_rgba(255, 50, 50, arena_warning), renderer->XtoRX(5));
        }

        if (Game_Event == GAME_EVENT_SIDE_LASER) {
            cy = renderer->YtoRY(arena_laser_y*BLOCK_W + BLOCK_W/2);
            al_draw_filled_rectangle(renderer->Get_Width()/2-renderer->XtoRX(arena_width/2), cy, renderer->Get_Width()/2+renderer->XtoRX(arena_width/2), cy+renderer->YtoRY(BLOCK_W), al_map_rgba(255, 50, 50, arena_warning/2));

            al_draw_filled_circle(renderer->Get_Width()/2-renderer->XtoRX(arena_width/2), cy+renderer->YtoRY(BLOCK_W/2), renderer->YtoRY(arena_laser_charge), al_map_rgba(200+arena_warning/4, 250, 100, 175));
        }

        if (Game_Event == GAME_EVENT_FLOOR_SPIKES) {
            al_draw_filled_rectangle(renderer->Get_Width()/2-renderer->XtoRX(arena_width/2), renderer->YtoRY(990), renderer->Get_Width()/2+renderer->XtoRX(arena_width/2), renderer->YtoRY(1040), al_map_rgba(255, 50, 50, arena_warning/2));
        }


        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 100+arena_warning/4), renderer->Get_Width()/2+renderer->XtoRX(4), renderer->YtoRY(304), ALLEGRO_ALIGN_CENTRE, arena_warning_string_1.c_str());
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 100+arena_warning/4), renderer->Get_Width()/2+renderer->XtoRX(4), renderer->YtoRY(364), ALLEGRO_ALIGN_CENTRE, arena_warning_string_2.c_str());
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 100+arena_warning/4), renderer->Get_Width()/2+renderer->XtoRX(4), renderer->YtoRY(424), ALLEGRO_ALIGN_CENTRE, arena_warning_string_3.c_str());

        al_draw_textf(warning_font, al_map_rgba(255, 0, 0, 100+arena_warning/2), renderer->Get_Width()/2, renderer->YtoRY(300), ALLEGRO_ALIGN_CENTRE, arena_warning_string_1.c_str());
        al_draw_textf(warning_font, al_map_rgba(255, 0, 0, 100+arena_warning/2), renderer->Get_Width()/2, renderer->YtoRY(360), ALLEGRO_ALIGN_CENTRE, arena_warning_string_2.c_str());
        al_draw_textf(warning_font, al_map_rgba(255, 0, 0, 100+arena_warning/2), renderer->Get_Width()/2, renderer->YtoRY(420), ALLEGRO_ALIGN_CENTRE, arena_warning_string_3.c_str());
    }

    if ((arena_laser_charge > 0) && (arena_laser_x != 0) && (Game_Event != GAME_EVENT_TOP_LASER) && (Game_Event != GAME_EVENT_SIDE_LASER)) {
        cx = renderer->Get_Width()/2 - renderer->XtoRX(480) + renderer->XtoRX(arena_laser_x*BLOCK_W + BLOCK_W/2 - arena_laser_charge/2);
        al_draw_filled_rectangle(cx, renderer->YtoRY(40), cx+renderer->XtoRX(arena_laser_charge), renderer->YtoRY(1040), al_map_rgba(255, 250, 100, arena_laser_charge*4));
    }

    if ((arena_laser_charge > 0) && (arena_laser_y != 0) && (Game_Event != GAME_EVENT_TOP_LASER) && (Game_Event != GAME_EVENT_SIDE_LASER)) {
        cy = renderer->YtoRY(arena_laser_y*BLOCK_W + BLOCK_W);
        al_draw_filled_rectangle(renderer->Get_Width()/2-renderer->XtoRX(arena_width/2), cy-renderer->YtoRY(arena_laser_charge), renderer->Get_Width()/2+renderer->XtoRX(arena_width/2), cy+renderer->YtoRY(arena_laser_charge), al_map_rgba(255, 250, 100, arena_laser_charge*4));
    }


    al_draw_scaled_bitmap(arena_border_v, 0, 0,
    al_get_bitmap_width(arena_border_v), al_get_bitmap_height(arena_border_v),
    renderer->Get_Width()/2 - renderer->XtoRX(arena_width/2+10), renderer->YtoRY(40),
    renderer->XtoRX(10), renderer->YtoRY(1000), 0);


    al_draw_scaled_bitmap(arena_border_v, 0, 0,
    al_get_bitmap_width(arena_border_v), al_get_bitmap_height(arena_border_v),
    renderer->Get_Width()/2 + renderer->XtoRX(arena_width/2), renderer->YtoRY(40),
    renderer->XtoRX(10), renderer->YtoRY(1000), ALLEGRO_FLIP_HORIZONTAL);

    al_draw_scaled_bitmap(arena_border_h, 0, 0,
    al_get_bitmap_width(arena_border_h), al_get_bitmap_height(arena_border_h),
    renderer->Get_Width()/2 - renderer->XtoRX(arena_width/2), renderer->YtoRY(40),
    renderer->XtoRX(arena_width), renderer->YtoRY(10), 0);

    al_draw_scaled_bitmap(arena_border_h, 0, 0,
    al_get_bitmap_width(arena_border_h), al_get_bitmap_height(arena_border_h),
    renderer->Get_Width()/2 - renderer->XtoRX(arena_width/2), renderer->YtoRY(1030),
    renderer->XtoRX(arena_width), renderer->YtoRY(10), 0);

    ALLEGRO_COLOR health_color;
    health_color=al_map_rgba(255, 255, 75, 225);
    if (player_health >= 100) health_color=al_map_rgba(175, 255, 75, 225);
    if (player_health >= 200) health_color=al_map_rgba(75, 255, 200, 225);
    if (player_health >= 300) health_color=al_map_rgba(125, 200, 255, 225);
    if (player_health >= 400) {
        health_color=al_map_rgba(125, 255, 200, 225);
        if (rand()%3) health_color=al_map_rgba(255, 255, 0, 225);
        if (rand()%3) health_color=al_map_rgba(255, 0, 125, 225);
        if (rand()%3) health_color=al_map_rgba(125, 125, 255, 225);
    }

    al_draw_filled_rectangle(renderer->Get_Width()/2 - renderer->XtoRX(480), renderer->YtoRY(1060), renderer->Get_Width()/2 - renderer->XtoRX(480) + renderer->XtoRX(960*player_health/400.0), renderer->YtoRY(1075), health_color);
    al_draw_rectangle(renderer->Get_Width()/2 - renderer->XtoRX(480), renderer->YtoRY(1060), renderer->Get_Width()/2 + renderer->XtoRX(480), renderer->YtoRY(1075), al_map_rgba(255, 255, 255, 225), 2);

    if (Game_Mode == GAME_MODE_INFINITE) al_draw_textf(score_font, al_map_rgb(255, 255, 255), renderer->Get_Width()/2 - renderer->XtoRX(320), renderer->YtoRY(2), 0, "SCORE: %5d", score);
    else al_draw_textf(score_font, al_map_rgb(255, 255, 255), renderer->Get_Width()/2 - renderer->XtoRX(320), renderer->YtoRY(2), 0, "REMAINING MOVES: %5d", remaining_moves);

    al_draw_textf(score_font, al_map_rgb(255, 255, 255), renderer->Get_Width()/2 + renderer->XtoRX(320), renderer->YtoRY(2), ALLEGRO_ALIGN_RIGHT, "COMBO: x%d", total_combo_multiplier);


    if (combo_string_active) {
        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, combo_opacity), renderer->Get_Width()/2+renderer->XtoRX(4), renderer->YtoRY(combo_string_y+4), ALLEGRO_ALIGN_CENTRE, combo_string.c_str());
        al_draw_textf(achievement_font, al_map_rgba(0, 255, 255, combo_opacity), renderer->Get_Width()/2, renderer->YtoRY(combo_string_y), ALLEGRO_ALIGN_CENTRE, combo_string.c_str());
    }

    if (player_health <= 0) {

        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2-renderer->XtoRX(5), renderer->YtoRY(600), ALLEGRO_ALIGN_CENTRE, "GAME OVER");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2-renderer->XtoRX(5), renderer->YtoRY(700), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2-renderer->XtoRX(5), renderer->YtoRY(800), ALLEGRO_ALIGN_CENTRE, "ESC for menu.");

        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2+renderer->XtoRX(5), renderer->YtoRY(600), ALLEGRO_ALIGN_CENTRE, "GAME OVER");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2+renderer->XtoRX(5), renderer->YtoRY(700), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2+renderer->XtoRX(5), renderer->YtoRY(800), ALLEGRO_ALIGN_CENTRE, "ESC for menu.");

        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(595), ALLEGRO_ALIGN_CENTRE, "GAME OVER");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(695), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(795), ALLEGRO_ALIGN_CENTRE, "ESC for menu.");

        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(605), ALLEGRO_ALIGN_CENTRE, "GAME OVER");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(705), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(805), ALLEGRO_ALIGN_CENTRE, "ESC for menu.");


        al_draw_textf(achievement_font, al_map_rgba(255, 255, 255, 255), renderer->Get_Width()/2, renderer->YtoRY(600), ALLEGRO_ALIGN_CENTRE, "GAME OVER");
        al_draw_textf(warning_font, al_map_rgba(255, 255, 255, 255), renderer->Get_Width()/2, renderer->YtoRY(700), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(255, 255, 255, 255), renderer->Get_Width()/2, renderer->YtoRY(800), ALLEGRO_ALIGN_CENTRE, "ESC for menu.");
    }

    bool clean=true;
    for (int x=0; x<12;x++) {
        for (int y=0; y<12;y++) {
            if (arena_blocks[x][y]) {
                clean=false;
                break;
            }
        }
    }

    if (remaining_moves <= 0 && falling_blocks.size()==0 && !clean) {
        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2-renderer->XtoRX(5), renderer->YtoRY(600), ALLEGRO_ALIGN_CENTRE, "NO MORE MOVES");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2-renderer->XtoRX(5), renderer->YtoRY(700), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2-renderer->XtoRX(5), renderer->YtoRY(800), ALLEGRO_ALIGN_CENTRE, "There's no ESC key...");

        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2+renderer->XtoRX(5), renderer->YtoRY(600), ALLEGRO_ALIGN_CENTRE, "NO MORE MOVES");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2+renderer->XtoRX(5), renderer->YtoRY(700), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2+renderer->XtoRX(5), renderer->YtoRY(800), ALLEGRO_ALIGN_CENTRE, "There's no ESC key...");

        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(595), ALLEGRO_ALIGN_CENTRE, "NO MORE MOVES");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(695), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(795), ALLEGRO_ALIGN_CENTRE, "There's no ESC key...");

        al_draw_textf(achievement_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(605), ALLEGRO_ALIGN_CENTRE, "NO MORE MOVES");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(705), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(0, 0, 0, 255), renderer->Get_Width()/2, renderer->YtoRY(805), ALLEGRO_ALIGN_CENTRE, "There's no ESC key...");


        al_draw_textf(achievement_font, al_map_rgba(255, 255, 255, 255), renderer->Get_Width()/2, renderer->YtoRY(600), ALLEGRO_ALIGN_CENTRE, "NO MORE MOVES");
        al_draw_textf(warning_font, al_map_rgba(255, 255, 255, 255), renderer->Get_Width()/2, renderer->YtoRY(700), ALLEGRO_ALIGN_CENTRE, "R to retry.");
        al_draw_textf(warning_font, al_map_rgba(255, 255, 255, 255), renderer->Get_Width()/2, renderer->YtoRY(800), ALLEGRO_ALIGN_CENTRE, "There's no ESC key...");
    }


    if (on_screen_tip_opacity > 0) {
        al_draw_filled_rectangle(renderer->Get_Width()/2-renderer->XtoRX(550), renderer->YtoRY(120), renderer->Get_Width()/2+renderer->XtoRX(550), renderer->YtoRY(300), al_map_rgba(0,0,0,on_screen_tip_opacity*0.55));

        al_draw_textf(tip_font, al_map_rgba(255, 255, 255, on_screen_tip_opacity*0.9), renderer->Get_Width()/2, renderer->YtoRY(130), ALLEGRO_ALIGN_CENTRE, on_screen_tip_string_1.c_str());
        al_draw_textf(tip_font, al_map_rgba(255, 255, 255, on_screen_tip_opacity*0.9), renderer->Get_Width()/2, renderer->YtoRY(230), ALLEGRO_ALIGN_CENTRE, on_screen_tip_string_2.c_str());
    }

    if (white_flash) al_draw_filled_rectangle(0, 0, renderer->Get_Width(), renderer->Get_Height(), al_map_rgba(255,255,255,white_flash));
}


void Load_Level(string name) {
    char str[256];
    FILE *fp;
    std::vector<std::string> text;
    fp = fopen(name.c_str(), "r+");
    int x, y;
    y=0;
    while (fgets(str, 256, fp) != NULL) {
        if (y == 13) {
            remaining_moves = ToFloat(str);
            break;
        }
        else if (y == 12) {
            new_arena_width = ToFloat(str);
            arena_width = ToFloat(str);
        }
        else {
            for (int x=0; x<12; x++) {
                if (str[x]=='P') {
                    player_x = x*BLOCK_W + BLOCK_W/2;
                    player_y = y*BLOCK_W+BLOCK_W/2;
                }
                else {
                    int t;
                    char s_char[3];
                    strcpy(s_char, "A");
                    s_char[0] = str[x];
                    t = ToFloat(s_char)-1;
                    if (t>=0) {
                        VObject *block;
                        float rx, ry;
                        int color;
                        int index;
                        index = x;
                        rx = renderer->RXtoX(renderer->Get_Width()/2)-BLOCK_W*6+(index)*BLOCK_W + BLOCK_W/2;
                        ry = y*BLOCK_W + BLOCK_W;
                        block = Create_Block(rx, ry, index, t);
                        arena_blocks[x][y] = block;
                        block->Set_Z(block->Get_X()+960-block->Get_Y());
                        game_vobjects.push_back(block);
                    }
                }
            }
        }
        y++;
        printf("\n");
    }
    fclose(fp);
}


void Create_Game() {
    Game_Created = true;

    game_vobjects.clear();
    pushed_blocks.clear();

    for (int x=0; x<12; x++) {
        for (int y=0; y<12; y++) {
            arena_blocks[x][y] = NULL;
        }
    }

    if (Game_Mode == GAME_MODE_PUZZLE) Load_Level(current_level);

    game_vobjects.push_back(background_bg1=new VObject(Get_Allegro_Bitmap("data/background.png"), 1920, 540, 1920, 1080));
    game_vobjects.push_back(background_bg2=new VObject(Get_Allegro_Bitmap("data/background.png"), 0, 540, 1920, 1080));

    arena_border_v = Get_Allegro_Bitmap("data/arena_border_vertical.png");
    arena_border_h = Get_Allegro_Bitmap("data/arena_border_horizontal.png");
    arena_bottom = Get_Allegro_Bitmap("data/bottom_block.png");


    game_vobjects.push_back(player=new VObject(Get_Allegro_Bitmap("data/character/stand/1.png"), 1000, 540));
    player->Set_Z(100000);

    game_vobjects.push_back(hat=new VObject(Get_Allegro_Bitmap("data/character/hats/" + ToString(rand()%5+1) + ".png"), 1000, 540));
    hat->Set_Z(100001);

    bottom_start_x = renderer->Get_Width()/2 - renderer->XtoRX(arena_width/2);
    while (bottom_start_x > 0) bottom_start_x -= renderer->XtoRX(BLOCK_W);

    float block_x=0;
    float render_x=0;
    while (bottom_start_x+(block_x*renderer->XtoRX(BLOCK_W)) < renderer->Get_Width()) {
        float cx=renderer->RXtoX(bottom_start_x)+block_x*BLOCK_W+BLOCK_W/2;
        game_vobjects.push_back(block=new VObject(arena_bottom, cx, 1040));
        block->Set_Z(1);

        game_vobjects.push_back(block=new VObject(Get_Allegro_Bitmap("data/spike.png"), cx, 985));
        block->Hide();
        spikes.push_back(block);
        block->Set_Z(2);

        block_x++;
    }


    beat = al_create_sample_instance(Get_Allegro_Sample("data/beat.ogg"));
    al_attach_sample_instance_to_mixer(beat, mixer);
    al_set_sample_instance_playmode(beat, ALLEGRO_PLAYMODE_LOOP);
    al_set_sample_instance_gain(beat, 1);
    al_play_sample_instance(beat);

    falling_blocks.clear();

    warning_font=al_load_font("data/font.ttf", (76.0/1080.0)*renderer->Get_Height(), 0);
    tip_font=al_load_font("data/font.ttf", (60.0/1080.0)*renderer->Get_Height(), 0);
    achievement_font=al_load_font("data/font.ttf", (120.0/1080.0)*renderer->Get_Height(), 0);
    score_font=al_load_font("data/font.ttf", (40.0/1080.0)*renderer->Get_Height(), 0);

    if (Game_Mode == GAME_MODE_INFINITE) {
        player_x = 480;
        player_y = 480;

        arena_width = 800;
        new_arena_width=800;
    }
    next_level_wait=120;

    used_tips.clear();

    first_tip = true;
}


void Reset_Game() {
    next_level_wait=120;
    white_flash=255;

    player_health=100;
    player->Show();
    hat->Show();

    hat->Set_Actual_Bit(Get_Allegro_Bitmap("data/character/hats/" + ToString(rand()%5+1) + ".png"));

    if (Game_Mode == GAME_MODE_INFINITE) {
        player_x = 480;
        player_y = 480;

        arena_width = 800;
        new_arena_width=800;
    }

    list<VObject *>::iterator it;
    for (it=falling_blocks.begin(); it!=falling_blocks.end(); it++) {
        Delete_From_List(*it);
    }
    falling_blocks.clear();

    for (int x=0; x<12; x++) {
        for (int y=0; y<12; y++) {
            if (arena_blocks[x][y]) Delete_From_List(arena_blocks[x][y]);
            arena_blocks[x][y] = NULL;
        }
    }

    total_combo_multiplier=1;
    score = 0;

    Game_Time=0;
    arena_warning=0;
    arena_change_cooldown=0;
    arena_spikes=false;
    arena_laser_charge=0;

    new_beat_gain = 0.8;
    new_beat_speed = 1.0;
    beat_change_cooldown = 300;

    Game_Event = GAME_EVENT_NONE;
    Game_Event_Duration=0;

    for (it=spikes.begin(); it!=spikes.end(); it++) {
        (*it)->Hide();
    }

    if (Game_Mode == GAME_MODE_PUZZLE) Load_Level(current_level);

    arena_change_cooldown=1000;
    To_Reset=false;
}


void Game_Tick(int Last_Scene) {
    if (!Game_Created) Create_Game();
    if (To_Reset) Reset_Game();
    Update_Game();
    Draw_Game();
}
