#include <allegro5/allegro.h>
#include <allegro5/allegro_primitives.h>
#include <allegro5/allegro_image.h>
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_audio.h>
#include <allegro5/allegro_acodec.h>
#include <stdio.h>
#include "config.h"
#include "game.h"
#include "inputlib.h"

const float FPS = 60;
int Scene=SCENE_MENU;

Renderer *renderer=NULL;

ALLEGRO_VOICE *voice;
ALLEGRO_MIXER *mixer;

int main(void)
{
    ALLEGRO_DISPLAY *display = NULL;
    ALLEGRO_EVENT_QUEUE *event_queue = NULL;
    ALLEGRO_TIMER *timer = NULL;
    bool redraw = true;

    srand(time(0));

    if(!al_init()) {
        printf("failed to initialize allegro!\n");
        return -1;
    }

    timer = al_create_timer(1.0 / FPS);
    if(!timer) {
        printf("failed to create timer!\n");
        return -1;
    }




    Config *config_file;
    config_file = new Config(DEFAULT_CONFIGURATION_PATH);
    if (!config_file) {
        printf("failed to create display!\n");
        return 0;
    }

    int win_mode=config_file->Get_Float("Display", "Mode", 2);
    if (win_mode==2) al_set_new_display_flags(al_get_new_display_flags() | ALLEGRO_FULLSCREEN_WINDOW);
    else if (win_mode==1) al_set_new_display_flags(al_get_new_display_flags() | ALLEGRO_WINDOWED);
    else al_set_new_display_flags(al_get_new_display_flags() | ALLEGRO_FULLSCREEN);

    display = al_create_display(config_file->Get_Float("Display", "Width", 800),
                                config_file->Get_Float("Display", "Height", 600));
    if(!display) {
        printf("failed to create display!\n");
        al_destroy_timer(timer);
        delete config_file;
        return -1;
    }

    al_init_image_addon();
    al_init_primitives_addon();
    al_init_font_addon();
    al_init_ttf_addon();
    al_init_acodec_addon();
    al_install_audio();
    al_install_keyboard();
    al_install_mouse();

    voice = al_create_voice(44100, ALLEGRO_AUDIO_DEPTH_INT16, ALLEGRO_CHANNEL_CONF_2);
    if (!voice) {
        printf("Could not create ALLEGRO_VOICE.\n");
        return 1;
    }

    mixer = al_create_mixer(44100, ALLEGRO_AUDIO_DEPTH_FLOAT32, ALLEGRO_CHANNEL_CONF_2);
    if (!mixer) {
        printf("al_create_mixer failed.\n");
        return 1;
    }

    al_attach_mixer_to_voice(mixer, voice);

    al_reserve_samples(16);

    renderer = new Renderer(display, 1920, 1080);

    Init_Mapping();
    Clear_Keyboard();

    event_queue = al_create_event_queue();
    if(!event_queue) {
        printf("failed to create event_queue!\n");
        al_destroy_display(display);
        al_destroy_timer(timer);
        return -1;
    }

    al_register_event_source(event_queue, al_get_display_event_source(display));
    al_register_event_source(event_queue, al_get_timer_event_source(timer));

    al_clear_to_color(al_map_rgb(0,0,0));
    al_set_window_title(display, "Blocky Rhythm");

    al_flip_display();

    al_start_timer(timer);

    float circle_x=0.0;

    int Last_Scene=-1;

    while(1)
    {
        bool quit=false;
        ALLEGRO_EVENT ev;
        al_wait_for_event(event_queue, &ev);

        if(ev.type == ALLEGRO_EVENT_TIMER) {
            al_clear_to_color(al_map_rgb(0,0,0));

            Poll_Input(display);

            switch (Scene) {
                case SCENE_QUIT :
                    quit=true;
                    break;
                case SCENE_MENU :
                    Menu_Tick(Last_Scene);
                    break;
                case SCENE_GAME :
                    Game_Tick(Last_Scene);
                    break;
            }

            Last_Scene = Scene;
            al_flip_display();
        }
        else if(ev.type == ALLEGRO_EVENT_DISPLAY_CLOSE) {
            break;
        }

        if (quit) break;
    }

    al_destroy_timer(timer);
    al_destroy_display(display);
    al_destroy_event_queue(event_queue);

    return 0;
}
