#include "renderer.h"
#include "stringlib.h"
#include "bitmap_manager.h"


Renderer::Renderer(ALLEGRO_DISPLAY *disp, float int_w, float int_h, float w, float h, float tx, float ty) {
    display = disp;
    assert(display);

    internal_w = int_w;
    internal_h = int_h;

    width = w;
    height = h;

    x = tx;
    y = ty;

    zoom = 1.0;


    if (width <= 0) width = al_get_display_width(disp);
    if (height <= 0) height = al_get_display_height(disp);

    aspect_ratio = width/height;
}


void Renderer::Sort_Z_List(list<VObject *> *vobjects) {
    list<VObject *> zl;
    zl.clear();

    for (list<VObject *>::iterator it=vobjects->begin(); it!=vobjects->end(); it++) {
        bool f=false;
        for (list<VObject *>::iterator it2=zl.begin(); it2!=zl.end(); it2++) {
            if ((*it)->Get_Z() < (*it2)->Get_Z()) {
                zl.insert(it2, (*it));
                f = true;
                break;
            }
        }
        if (!f) zl.push_back((*it));
    }

    (*vobjects) = zl;
}

void Renderer::Render(list<VObject *> *vobjects, float camera_x, float camera_y) {
    Sort_Z_List(vobjects);

    al_set_blender(ALLEGRO_ADD, ALLEGRO_ALPHA, ALLEGRO_INVERSE_ALPHA);

    for (list<VObject *>::iterator it=vobjects->begin(); it!=vobjects->end(); it++) {
        if (!(*it)->Is_Visible()) continue;
        float dx, dy, dw, dh;
        dx = XtoRX((*it)->Get_X()-camera_x)+x;
        dy = YtoRY((*it)->Get_Y()-camera_y)+y;

        dw = XtoRX((*it)->Get_W()) / (float) al_get_bitmap_width((*it)->Get_Actual_Bit());
        dh = YtoRY((*it)->Get_H()) / (float) al_get_bitmap_height((*it)->Get_Actual_Bit());

        al_draw_tinted_scaled_rotated_bitmap((*it)->Get_Actual_Bit(), (*it)->Get_Tint(),
        (*it)->Get_CX(), (*it)->Get_CY(), dx, dy, dw, dh, (*it)->Get_Angle(), (*it)->Get_Flags());

        if ((*it)->Get_Break_Opacity() > 0) {
            al_draw_tinted_scaled_rotated_bitmap(Get_Allegro_Bitmap("data/break.png"), al_map_rgba(255, 255, 255, (*it)->Get_Break_Opacity()),
            (*it)->Get_CX(), (*it)->Get_CY(), dx, dy, dw, dh, (*it)->Get_Angle(), (*it)->Get_Flags());
        }
    }

}


/*
  Lots of conversion routines between different coordinates.

  X-Y: These represent the internal X and y coordinates, used as the "screen" space. These,
  are in pixel-perfect positions related to the internal_w and h of the renderer.

  That means that if the internal_h of the renderer is set to 100, a sprite with a height of 10 would
  take a 10% of the vertical space on the screen.

  GX - GY: These are the real-screen positions, but they aren't affected by the renderer's zoom.
  Used by GUIs normally.

  RX - RY: Real-screen positions, affected by zoom

  SX - SY: Real-screen positions, that are unaffected by the aspect ratio of the screen.
*/


float Renderer::XtoGX(float v) {
  return v * ((float)width / ((internal_h) * aspect_ratio));
}

float Renderer::YtoGY(float v) {
  return v * ((float)height / (internal_h));
}

float Renderer::GXtoX(float v) {
  return v / ((float)width / ((internal_h) * aspect_ratio));
}

float Renderer::GYtoY(float v) {
  return v / ((float)height / (internal_h));
}


float Renderer::XtoRX(float v) {
  return v * ((float)width*zoom / ((internal_h) * aspect_ratio));
}

float Renderer::YtoRY(float v) {
  return v * ((float)height*zoom / (internal_h));
}

float Renderer::RXtoX(float v) {
  return v / ((float)width*zoom / ((internal_h) * aspect_ratio));
}

float Renderer::RYtoY(float v) {
  return v / ((float)height*zoom / (internal_h));
}


float Renderer::XtoSX(float v) {
  return (v/internal_w) * width;
}

float Renderer::YtoSX(float v) {
  return (v/internal_h) * height;
}
